<?php
/**
 * @package     Joomla.Administrator
 * @subpackage  com_nicepage
 */

defined('_JEXEC') or die;

/**
 * Class NicepageHelpersNicepage
 */
class NicepageHelpersNicepage
{
    /**
     * Extension type name
     *
     * @var string
     */
    public static $extension = 'com_nicepage';

    /**
     * Get actions for nicepage component
     *
     * @return JObject
     */
    public static function getActions()
    {
        $user = JFactory::getUser();
        $result = new JObject;

        $assetName = 'com_nicepage';
        $level = 'component';

        $actions = JAccess::getActions('com_nicepage', $level);

        foreach ($actions as $action) {
            $result->set($action->name, $user->authorise($action->name, $assetName));
        }

        return $result;
    }

    /**
     * Check site ssl or not
     *
     * @return bool
     */
    public static function isSSL()
    {
        $isSSL = false;

        if (isset($_SERVER['HTTP_X_FORWARDED_PROTO']) && $_SERVER['HTTP_X_FORWARDED_PROTO'] == 'https') {
            $_SERVER['HTTPS'] = 'on';
        }

        if (isset($_SERVER['HTTPS'])) {
            if ('on' == strtolower($_SERVER['HTTPS'])) {
                $isSSL = true;
            }
            if ('1' == $_SERVER['HTTPS']) {
                $isSSL = true;
            }
        } elseif (isset($_SERVER['SERVER_PORT']) && ('443' == $_SERVER['SERVER_PORT'])) {
            $isSSL = true;
        }
        return $isSSL;
    }

    /**
     * Get domain get parameter
     *
     * @return mixed|string
     */
    public static function getDomainParam()
    {
        $domain = isset($_GET['domain'])
            ? urldecode($_GET['domain'])
            : (defined('NICEPAGE_DOMAIN') ? NICEPAGE_DOMAIN : '');

        $domain = preg_replace('#^https?:#', '', $domain); // remove protocol
        $domain = preg_replace('#\/$#', '', $domain); // remove last slash
        return $domain;
    }

    /**
     * Get site url
     *
     * @return string
     */
    public static function getSiteUrl()
    {
        return dirname(dirname((JURI::current())));
    }

    /**
     * Get files for nicepage starting
     *
     * @return array
     */
    public static function getStartFiles()
    {
        $hash = md5(round(microtime(true)));
        $domain = self::getDomainParam();

        return array(
            'editor' => self::getSiteUrl() . '/administrator/components/com_nicepage/assets/js/editor.js?version=' . $hash,
            'loader' => $domain ? $domain . '/Editor/loader.js' : self::getLoader($hash)
        );
    }

    /**
     * Get custom loader file
     *
     * @param string $hash Hash for get parameter
     *
     * @return string
     */
    public static function getLoader($hash) {
        return self::getSiteUrl() . '/administrator/components/com_nicepage/assets/app/loader.js?version=' . $hash;
    }

    /**
     * Get actions list for nicepage app
     *
     * @return array
     */
    public static function getEditorSettings()
    {
        $current = dirname(dirname((JURI::current())));
        $index = $current . '/administrator/index.php?option=com_nicepage&controller=actions';
        return array(
            'actions' => array(
                'uploadImage' => $index . '&action=uploadImage',
                'savePage' => $index . '&action=savePage',
                'getSite' => $index . '&action=getSite',
                'getSitePosts' => $index . '&action=getSitePosts',
                'getPage' => $index . '&action=getPage',
                'getManifest' => $index . '&action=getManifest',
                'saveSiteSettings' => $index . '&action=saveSiteSettings',
            ),
            'uploadImageOptions' => array(
                'formFileName' => 'async-upload'
            ),
            'dashboardUrl' => $current . '/administrator/',
            'editPostUrl' => $current . '/administrator/index.php?option=com_content&view=article&layout=edit&id={id}'
        );
    }

    /**
     * Get cms custom settings
     *
     * @return array
     */
    public static function getCmsSettings()
    {
        return array(
            'defaultImageUrl' => dirname(dirname((JURI::current()))) . '/administrator/components/com_nicepage/assets/images/nicepage-images/default-image.jpg',
            'isFirstStart' => false
        );
    }

    /**
     * Get nicepage properties
     *
     * @return mixed
     */
    public static function getConfig()
    {
        $db = JFactory::getDBO();
        $query = $db->getQuery(true);
        $query->select('params')->from('#__nicepage_params')->where('name=' . $query->quote('com_nicepage'));
        $db->setQuery($query);
        $result = $db->loadResult();

        if (!$result) {
            return array();
        }

        $registry = new JRegistry();
        $registry->loadString($db->loadResult());
        $ret = $registry->toArray();

        if (isset($ret['siteStyleCss']) && substr($ret['siteStyleCss'], 0, 6) === '<style') {
            // backward compatibility
            $css = preg_replace('#</?style[^>]*>#', '', $ret['siteStyleCss']);
            ob_start();
            include_once dirname(JPATH_PLUGINS) . '/administrator/components/com_nicepage/assets/css/nicepage-dynamic.css';
            $dynamicPart = ob_get_clean();
            $ret['siteStyleCss'] = $css . $dynamicPart;
        }
        return $ret;
    }

    /**
     * Save nicepage settings
     *
     * @param array $data Data parameters
     *
     * @return mixed|string
     */
    public static function saveConfig($data) {
        $db = JFactory::getDBO();
        $query = $db->getQuery(true);
        $query->select('params')->from('#__nicepage_params')->where('name=' . $query->quote('com_nicepage'));
        $db->setQuery($query);

        $registry = new JRegistry();


        $registry->loadString($db->loadResult());
        $params = $registry->toArray();

        $excludeParameters = array('option', 'action', 'controller');
        foreach ($data as $key => $value) {
            if (in_array($key, $excludeParameters)) {
                continue;
            }
            $params[$key] = $value;
        }

        $registry->loadArray($params);
        $str = $registry->toString();

        $query = $db->getQuery(true);
        $query->update('#__nicepage_params')->set(
            $db->quoteName('params') . '=' .
            $db->quote($str)
        )->where('name=' . $query->quote('com_nicepage'));
        $db->setQuery($query);
        $db->query();
    }

}