<?php
/**
 * @package     Joomla.Administrator
 * @subpackage  com_nicepage
 */

defined('_JEXEC') or die;

/**
 * Class NicepageModelsSections
 */
class NicepageModelsSections
{

    /**
     * @param string $content Page content
     *
     * @return mixed
     */
    public static function processSectionsHtml($content) {
        $content = NicepageModelsSections::processDefaultImage($content);
        $content = NicepageModelsSections::processForm($content);
        $content = NicepageModelsSections::processPositions($content);
        return $content;
    }

    /**
     * @var array All blocks on the page
     */
    public static $blockLayouts = array();

    /**
     * Process all positions on the page
     *
     * @param string $content Page content
     *
     * @return mixed
     */
    public static function processPositions($content) {
        $content = preg_replace_callback(
            '/<\!--position-->([\s\S]+?)<\!--\/position-->/',
            function ($match) {
                $block = $match[1];
                preg_match('/data-position="([^"]*)"/', $block, $match2);
                $position = $match2[1];
                $i = count(NicepageModelsSections::$blockLayouts) + 1;
                NicepageModelsSections::$blockLayouts[$i] = $block;
                $document = JFactory::getDocument();
                if ($position && $document->countModules($position) !== 0) {
                    $attr = array(
                        'style' => 'upstylefromplugin',
                        'iterator' => $i,
                        'title' => 'name-' . $i,
                        'id' => $i,
                        'name' => $position
                    );
                    include_once dirname(dirname(__FILE__)) . '/helpers/modules.php';
                    return $document->getBuffer('modules', $position, $attr);
                } else {
                    return '';
                }
            },
            $content
        );
        return $content;
    }

    /**
     * Processing of default image path
     *
     * @param string $content Page content
     *
     * @return mixed
     */
    public static function processDefaultImage($content)
    {
        // replace default image placeholder
        $url = JURI::current() . 'administrator/components/com_nicepage/assets/images/nicepage-images/default-image.jpg';
        return str_replace('[image_default]', $url, $content);
    }

    /**
     * Processing of forms
     *
     * @param string $content Page content
     *
     * @return mixed
     */
    public static function processForm($content)
    {
        $content = preg_replace('/(<form[^>]*action=[\'\"]+).*?([\'\"][^>]*source=[\'\"]joomla)/', '$1index.php?option=com_nicepage&task=sendmail$2', $content);
        $content = preg_replace_callback(
            '/<form[^>]+redirect-address=[\'\"](.*?)[\'\"][^>]*>/',
            function ($matches) {
                if (preg_match('/<form[^>]+redirect=[\'\"]true[\'\"]/', $matches[0])) {
                    return $matches[0] . '<input type="hidden" name="redirect" value="' . $matches[1] . '">';
                }
                return $matches[0];
            },
            $content
        );
        return $content;
    }

    /**
     * @param int $pageId Page id
     *
     * @return null
     */
    public static function getSectionsObject($pageId)
    {
        $db = JFactory::getDBO();
        $query = $db->getQuery(true);
        $query->select('*');
        $query->from('#__nicepage_sections');
        $query->where('page_id = ' . $pageId);
        $db->setQuery($query);
        $list = $db->loadObjectList();
        if (count($list) > 0) {
            return $list[0];
        } else {
            return null;
        }
    }

    /**
     * Clear preview page
     *
     * @param object $sectionsObject sections page object
     *
     * @return null
     */
    public static function clearPreview($sectionsObject) {
        $props = unserialize(base64_decode($sectionsObject->props));
        $props['preview'] = '';

        $db = JFactory::getDBO();
        $query = $db->getQuery(true);
        $query->update('#__nicepage_sections');
        $query->set($db->quoteName('props') . '=' . $db->quote(base64_encode(serialize($props))));
        $query->where('page_id=' . $query->escape($sectionsObject->page_id));
        $db->setQuery($query);
        $db->query();
    }

    /**
     * @return mixed
     */
    public static function getAllPageIds()
    {
        $db = JFactory::getDBO();
        $query = $db->getQuery(true);
        $query->select('page_id');
        $query->from('#__nicepage_sections');
        $db->setQuery($query);
        return $db->loadAssocList(null, 'page_id');
    }

    /**
     * @param object $sectionsObj sections page object
     *
     * @return mixed|string
     */
    public static function getSectionsScreenshots($sectionsObj)
    {
        if (!$sectionsObj) {
            return '';
        }

        $props = unserialize(base64_decode($sectionsObj->props));
        $publishHtml = isset($props['publishHtml']) ? $props['publishHtml'] : '';
        $bodyClass = isset($props['bodyClass']) ? $props['bodyClass'] : '';
        $head = isset($props['head']) ? $props['head'] : '';
        $fonts = isset($props['fonts']) ? $props['fonts'] : '';
        $publishHtml = NicepageModelsSections::processSectionsHtml($publishHtml);
        preg_match_all('/<section[\s\S]+?<\/section>/', $publishHtml, $matches, PREG_SET_ORDER);
        $count = count($matches);
        if ($count > 4) {
            for ($i = 4; $i < $count; $i++) {
                $publishHtml = str_replace($matches[$i], '', $publishHtml);
            }
        }
        $assets = '/administrator/components/com_nicepage/assets';
        $nicepageCss = JURI::root(true) . $assets. '/css/nicepage.css';

        include_once  dirname(JPATH_PLUGINS) . '/administrator/components/com_nicepage/helpers/nicepage.php';
        $params = NicepageHelpersNicepage::getConfig();
        $siteStyleCss = isset($params['siteStyleCss']) ? $params['siteStyleCss'] : '';

        $ret = <<<EOF
<!DOCTYPE html>
<html>        
    <head>
    <style>
        body {
            cursor: pointer;
        }
    </style>
    <link rel="stylesheet" href="$nicepageCss">
    $fonts
    <style>
    	$siteStyleCss
        $head
    </style>
    </head>
    <body class="$bodyClass">
        $publishHtml
    </body>
</html>
EOF;
        return json_encode($ret);
    }
}