<?php
/**
 * @package     Joomla.Plugin
 * @subpackage  Content.nicepage
 */

defined('_JEXEC') or die;

/**
 * Class PlgContentNicepage
 */
class PlgContentNicepage extends JPlugin
{

    private $_pageView = 'default';
    private $_isNicepageTheme = '0';
    private $_sectionsHtml = '';
    private $_backlink = '';
    private $_styles = '';

    /**
     * @param object $context Context
     * @param object $row     Row
     * @param array  $params  Parameters
     * @param int    $page    Page number
     */
    public function onContentPrepare($context, &$row, &$params, $page = 0)
    {
        $app = JFactory::getApplication();

        if ($context !== 'com_content.article' || $app->isAdmin() || !$this->_npInstalled()) {
            return;
        }

        if (isset($row->id)) {
            $npComponent = dirname(JPATH_PLUGINS) . '/administrator/components/com_nicepage';
            $secfp = $npComponent . '/models/sections.php';
            $helper = $npComponent . '/helpers/nicepage.php';
            if (!file_exists($secfp) || !file_exists($helper)) {
                return;
            }
            include_once $secfp;
            include_once $helper;

            $sectionsObj = NicepageModelsSections::getSectionsObject($row->id);
            if ($sectionsObj) {
                $row->nicepage = true;
                $config = NicepageHelpersNicepage::getConfig();
                $props = unserialize(base64_decode($sectionsObj->props));

                $this->_pageView = isset($props['pageView']) ? $props['pageView'] : '';
                $theme = $app->getTemplate(true);
                $this->_isNicepageTheme = $theme->params->get('nicepagetheme', '0');

                // Change live props to preview props
                $isPreview = JRequest::getBool('isPreview', false);
                $preview = isset($props['preview']) ? $props['preview'] : '';
                if ($isPreview && $preview !== '') {
                    $props = $props['preview'];
                }
                // Backlink processing
                $hideBacklink = isset($config['hideBacklink']) ? (bool)$config['hideBacklink'] : false;
                if ($hideBacklink) {
                    $props['backlink'] = str_replace('u-backlink', 'u-backlink u-hidden', $props['backlink']);
                }
                // Start shortcodes processing
                $currentText = $row->text;
                $row->text = $props['publishHtml'];
                if (class_exists('PlgContentLoadmodule')) {
                    $dispatcher = JEventDispatcher::getInstance();
                    $pluginParams = JPluginHelper::getPlugin('content', 'loadmodule');
                    $pluginObject = new PlgContentLoadmodule($dispatcher, (array)$pluginParams);
                    $pluginObject->onContentPrepare('content.loadmodule', $row, $params);
                }
                $props['publishHtml'] = $row->text;
                $row->text = $currentText;
                // End shortcodes processing

                // Redirect to landing page
                if ($this->_pageView == 'landing') {
                    $app->set('theme', 'landing');
                    $app->set('themes.base', JPATH_ADMINISTRATOR . '/components/com_nicepage/views');
                    $app->set('themeFile', 'landing.php');
                    $app->set('props', $props);
                    return;
                }

                $this->_addPageProperties($props);
                $this->_addScripts($config);
                $this->_setStyles($props, $config);
                $this->_setBacklink($props);
                $this->_setSectionsHtml($props);

                $row->introtext = $row->text = '<!--np_content-->' . $this->_getSectionsHtml() . '<!--/np_content-->';
            }
        }
        return;
    }

    /**
     *  Proccess content before rendering
     */
    public function onAfterRender()
    {
        $app = JFactory::getApplication();

        if ($app->isAdmin()) {
            return;
        }

        $pageContent = $app->getBody();

        if (preg_match('/<\!--np\_content-->([\s\S]+?)<\!--\/np\_content-->/', $pageContent, $matches)) {
            $sectionsHtml = $matches[1];
            if ($sectionsHtml && $this->_pageView === 'landing_with_header_footer') {
                $bodyStartTag = '<body>';
                $bodyContent = '';
                $bodyEndTag = '</body>';
                if (preg_match('/(<body[^>]+>)([\s\S]*)(<\/body>)/', $pageContent, $matches)) {
                    $bodyStartTag = $matches[1];
                    $bodyContent = trim($matches[2]);
                    $bodyEndTag = $matches[3];
                }

                if ($bodyContent == '') {
                    $newPageContent = $bodyStartTag . $sectionsHtml . $bodyEndTag;
                } else {
                    $newPageContent = $bodyStartTag;
                    if (preg_match('/<header[^>]+>[\s\S]*<\/header>/', $bodyContent, $matches2)) {
                        $newPageContent .= $matches2[0];
                    }
                    $newPageContent .= $sectionsHtml;
                    if (preg_match('/<footer[^>]+>[\s\S]*<\/footer>/', $bodyContent, $matches3)) {
                        $newPageContent .= $matches3[0];
                    }
                    $newPageContent .= $bodyEndTag;
                }
                $pageContent = preg_replace('/(<body[^>]+>)([\s\S]*)(<\/body>)/', $newPageContent, $pageContent);
            } else {
                $pageContent = preg_replace('/<!--\/?np\_content-->/', '', $pageContent);
            }
        }
        if (JFactory::getApplication()->input->get('toEdit', '0') === '1') {
            $pageContent = preg_replace('/class="(item-page|u-page-root)/', ' id="np-test-container" class="$1', $pageContent);
        }
        $pageContent = str_replace('</head>', $this->_getStyles() . '</head>', $pageContent);
        $pageContent = str_replace('</body>', $this->_getBacklink() . '</body>', $pageContent);
        $app->setBody($pageContent);
    }

    /**
     * Check nicepage component
     *
     * @return bool
     */
    private function _npInstalled()
    {
        if (!file_exists(dirname(JPATH_ADMINISTRATOR) . '/components/com_nicepage')) {
            return false;
        }
        if (!JComponentHelper::getComponent('com_nicepage', true)->enabled) {
            return false;
        }
        return true;
    }

    /**
     * Set sections property
     *
     * @param array $props Page properties
     */
    private function _setSectionsHtml($props)
    {
        $responsiveScript = <<<SCRIPT
<script>
    (function ($) {
        var ResponsiveCms = window.ResponsiveCms;
        if (!ResponsiveCms) {
            return;
        }
        ResponsiveCms.contentDom = $('script:last').parent();
    
        if (typeof ResponsiveCms.recalcClasses === 'function') {
            ResponsiveCms.recalcClasses();
        }
    })(jQuery);
</script>
SCRIPT;
        $this->_sectionsHtml = $responsiveScript . NicepageModelsSections::processSectionsHtml($props['publishHtml']);

        if ($this->_isNicepageTheme !== '1' && $this->_pageView == 'default') {
            $this->_sectionsHtml = '<div class="nicepage-container"><div class="'. $props['bodyClass'] .'">' . $this->_sectionsHtml . '</div></div>';
        } else {
            $bodyScript = <<<SCRIPT
<script>
var body = document.body;
    body.className += ' {$props['bodyClass']}';
</script>
SCRIPT;
            $this->_sectionsHtml = $bodyScript . $this->_sectionsHtml;
        }
    }

    /**
     * Get sections html
     *
     * @return string
     */
    private function _getSectionsHtml()
    {
        return $this->_sectionsHtml;
    }

    /**
     * Set backlink property
     *
     * @param array $props Page properties
     */
    private function _setBacklink($props)
    {
        $backlink = isset($props['backlink']) ?  $props['backlink'] : '';
        if ($backlink && $this->_pageView == 'default') {
            if ($this->_isNicepageTheme !== '1') {
                $backlink = '<div class="nicepage-container"><div class="'. $props['bodyClass'] .'">' . $backlink . '</div></div>';
            } else {
                $backlink = '';
            }
        }
        $this->_backlink = $backlink;
    }

    /**
     * Get page backlink
     *
     * @return string
     */
    private function _getBacklink()
    {
        return $this->_backlink;
    }

    /**
     * Add page properties
     *
     * @param array $props Page properties
     */
    private function _addPageProperties($props)
    {
        $document = JFactory::getDocument();
        if (isset($props['titleInBrowser']) && $props['titleInBrowser'] != '') {
            $document->setTitle($props['titleInBrowser']);
        }
        if (isset($props['description']) && $props['description'] != '') {
            $document->setDescription($props['description']);
        }
        if (isset($props['keywords']) && $props['keywords'] != '') {
            $document->setMetadata('keywords', $props['keywords']);
        }
        if (isset($props['metaTags']) && $props['metaTags'] != '') {
            $document->addCustomTag($props['metaTags']);
        }
        if (isset($props['customHeadHtml']) && $props['customHeadHtml'] != '') {
            $document->addCustomTag($props['customHeadHtml']);
        }
    }

    /**
     * Set style property
     *
     * @param array $props  Page properties
     * @param array $config Component configuration
     */
    private function _setStyles($props, $config)
    {
        $sectionsHead = $props['head'];
        $autoResponsive = isset($config['autoResponsive']) ? !!$config['autoResponsive'] : true;
        if ($autoResponsive) {
            $sectionsHead = preg_replace('#\/\*RESPONSIVE_MEDIA\*\/([\s\S]*?)\/\*\/RESPONSIVE_MEDIA\*\/#', '', $sectionsHead);
        } else {
            $sectionsHead = preg_replace('#\/\*RESPONSIVE_CLASS\*\/([\s\S]*?)\/\*\/RESPONSIVE_CLASS\*\/#', '', $sectionsHead);
        }

        if ($this->_isNicepageTheme !== '1') {
            $assets = '/administrator/components/com_nicepage/assets';
            if ($this->_pageView == 'default') {
                ob_start();
                include_once dirname(JPATH_PLUGINS) . $assets . '/css/nicepage.css';
                $sectionsHead .= ob_get_clean();
                $sectionsHead = preg_replace_callback(
                    '/([^{}]+)\{[^{}]+?\}/',
                    function ($match) {
                        $selectors = $match[1];
                        $parts = explode(',', $selectors);
                        $newSelectors = implode(
                            ',',
                            array_map(
                                function ($part) {
                                    return ' .nicepage-container ' . $part;
                                },
                                $parts
                            )
                        );
                        return str_replace($selectors, $newSelectors, $match[0]);
                    },
                    $sectionsHead
                );
            } else {
                if (file_exists(dirname(JPATH_PLUGINS) . $assets . '/css/nicepage.css')) {
                    $this->_styles .= '<link href="' . JURI::root(true) . $assets . '/css/nicepage.css" rel="stylesheet">';
                }
            }
        }

        $this->_styles .= isset($props['fonts']) ? $props['fonts'] : '';
        $publishNicePageCss = $config && isset($config['siteStyleCss']) ? ($config['siteStyleCss'] . "\r\n"): '';
        $this->_styles .= '<style>' . $publishNicePageCss . $sectionsHead . '</style>';
    }

    /**
     * Get styles property
     *
     * @return string
     */
    private function _getStyles()
    {
        return $this->_styles;
    }

    /**
     * Add scripts
     *
     * @param array $config Component configuration
     */
    private function _addScripts($config)
    {
        JHtml::_('jquery.framework');

        $document = JFactory::getDocument();
        $assets = '/administrator/components/com_nicepage/assets';
        if ($this->_isNicepageTheme !== '1' && isset($config['jquery']) && $config['jquery'] == '1') {
            $document->addScript(JURI::root(true) . $assets . '/js/jquery.js');
        }
        if ($this->_isNicepageTheme !== '1' && file_exists(dirname(JPATH_PLUGINS) . $assets . '/js/nicepage.js')) {
            $document->addScript(JURI::root(true) . $assets . '/js/nicepage.js');
        }
    }
}