<?php
/**
 * @package     Joomla.Plugin
 * @subpackage  Editors-xtd.nicepage
 */

defined('_JEXEC') or die;

class PlgButtonNicepage extends JPlugin
{
    private $_adminUrl = '';
    private $_domain = '';

    private $_article = null;
    private $_sections = null;
    private $_isConvertRequired = null;

    private $_scripts = '';
    /**
     * PlgButtonNicepage constructor.
     *
     * @param object $subject Subject object
     * @param object $config  Config object
     */
    public function __construct(& $subject, $config)
    {
        parent::__construct($subject, $config);
        $this->loadLanguage();

        if ($this->isDisplayed()) {
            $this->setAdminUrl();
            $this->setDomain();
            $aid = JFactory::getApplication()->input->get('id', '');
            $this->_sections = NicepageModelsSections::getSectionsObject($aid);
            $this->_article = JTable::getInstance("content");
            $this->_article->load($aid);
            $this->_isConvertRequired = !$this->_sections && ($this->_article->introtext . $this->_article->fulltext);
        }
    }

    /**
     * @param string $name Name
     */
    public function onDisplay($name)
    {
        if (!$this->_article) {
            return;
        }

        $this->addCommonScript();
        $this->addLinkDialogScript();
        $this->addDataBridgeScript();

        $this->addMainScript();

        $this->includeScripts();

        $this->displayEditButton();
        $this->displayScreenshots();
    }

    /**
     * Check editor button displaying
     *
     * @return bool
     */
    public function isDisplayed()
    {
        $app = JFactory::getApplication();
        $input = $app->input;
        $option = $input->get('option');
        $aid = $input->get('id', '');

        if (!in_array($option, array('com_content')) || '' == $aid || !$app->isAdmin()) {
            return false;
        }

        if (!$this->nicepageComponentInstalled()) {
            return false;
        }

        $nicepageComponentPath = dirname(JPATH_PLUGINS) . '/administrator/components/com_nicepage';
        $secfp      = $nicepageComponentPath . '/models/sections.php';
        $actionsfp  = $nicepageComponentPath . '/models/actions.php';
        $helperfp   = $nicepageComponentPath . '/helpers/nicepage.php';
        if (!file_exists($secfp) || !file_exists($actionsfp) || !file_exists($helperfp)) {
            return false;
        }
        include_once $secfp;
        include_once $actionsfp;
        include_once $helperfp;

        return true;
    }

    /**
     * Add main script
     */
    public function addMainScript()
    {
        $input = JFactory::getApplication()->input;
        $aid = $input->get('id', '');
        $start = $input->get('start', '0');
        $autostart = $input->get('autostart', '0');
        $savePageTypeUrl = $this->getAdminUrl() . '/index.php?option=com_nicepage&controller=actions&action=savePageType';

        $themeTypographyCacheForceRefresh = '0';
        $cachedDefaultTheme = isset($_COOKIE['DEFAULT_THEME']) ? $_COOKIE['DEFAULT_THEME'] : '';
        if (!$cachedDefaultTheme || $cachedDefaultTheme !== $this->getDefaultTemplate()) {
            setcookie('DEFAULT_THEME', $this->getDefaultTemplate(), time() + 31536000); // will expire after year
            $themeTypographyCacheForceRefresh = '1';
        }

        $pageView = 'landing';
        if ($this->_sections) {
            $props = unserialize(base64_decode($this->_sections->props));
            $pageView = isset($props['pageView']) ? $props['pageView'] : $pageView;
        }

        switch($pageView) {
        case 'landing':
            $templateOptions = JText::sprintf('PLG_EDITORS-XTD_TEMPLATE_OPTIONS', '', 'selected', '');
            break;
        case 'landing_with_header_footer':
            $templateOptions = JText::sprintf('PLG_EDITORS-XTD_TEMPLATE_OPTIONS', '', '', 'selected');
            break;
        default:
            $templateOptions = JText::sprintf('PLG_EDITORS-XTD_TEMPLATE_OPTIONS', 'selected', '', '');
        }
        $buttonText = $this->_isConvertRequired ? JText::_('PLG_EDITORS-XTD_TURN_TO_NICEPAGE_BUTTON_TEXT') : JText::_('PLG_EDITORS-XTD_EDIT_WITH_NICEPAGE_BUTTON_TEXT');
        $buttonAreaClass = $this->_isConvertRequired ? '' : 'nicepage-select-template-area';

        $this->_scripts .= <<<EOF

jQuery(function($) {
    
    var dataInfo = dataBridge.getInfo();
    if (!dataInfo['themeColorScheme']) {
        var loadCallback;
        var forceRefresh = '$themeTypographyCacheForceRefresh';
        var needResetCache = !localStorage.np_theme_typography_cache ||
            forceRefresh === '1';
    
        if (needResetCache) {
            delete localStorage.np_theme_typography_cache;
        }
        
        window.loadAppHook = function (load) {
            if (localStorage.np_theme_typography_cache) {
                jQuery.extend(dataBridgeData.info, JSON.parse(localStorage.np_theme_typography_cache));
                console.log('Regular load app.js');
                load();
                return;
            }
            loadCallback = load;
        };
        var iframe = $('<iframe>', { id: 'np-loader', 'style': 'position:absolute, visibility: hidden, width: 1800px; height:0; border:0;' });
        $('nav').before(iframe);
        
        var loaderIframe = document.getElementById('np-loader');
        loaderIframe.addEventListener("load", function() {
            localStorage.np_theme_typography_cache = JSON.stringify(NpTypographyParser.parse(loaderIframe));
            $(loaderIframe).remove();
            console.log('Typography cache updated');
            $.extend(dataBridgeData.info, JSON.parse(localStorage.np_theme_typography_cache));
            if (loadCallback) {
                console.log('Deferred load app.js');
                loadCallback();
            }
        });
        loaderIframe.src = dataBridgeData.pageUrlForParsing;
    }
            
    // autostart nicepage from cms admin main menu
    if ('$start' == '1' || '$autostart' == '1') {
        runNicepage();
    }
    var nicepageButton = $('<a href="#" class="btn nicepage-button">$buttonText</a>'),
        nicepageArea = $('<div class="$buttonAreaClass"></div>');

    nicepageArea.append(nicepageButton);
    $('form').before(nicepageArea);
    nicepageButton.click(function(e) {
        e.preventDefault();
        runNicepage();
    });
    if ('$buttonAreaClass' !== '') {
        var selectObj = $('$templateOptions');
        nicepageButton.after(selectObj);
        $('#toolbar-apply button, #toolbar-save button').click(function() {
            var pageType = $('.nicepage-select-template').val();
            sendRequest({
                url: '$savePageTypeUrl',
                method: 'POST',
                data: {
                    pageType: pageType,
                    pageId: $aid
                }
            }, function (error, response) {
                if (error) {
                    console.error(e);
                    alert('Save page type error.');
                }
            });
        });
    }
});
EOF;
    }

    /**
     * Add common scripts
     */
    public function addCommonScript()
    {
        $domain = $this->getDomain();
        $aid = JFactory::getApplication()->input->get('id', '');
        $currentUrl = $this->getAdminUrl() . '/index.php?option=com_nicepage&postid=' . $aid . ($domain ? '&domain=' . $domain : '');

        JHtml::_('behavior.modal'); // for SqueezeBox

        $this->_scripts .= <<<EOF
function runNicepage()
{
    (function($){
        var iframe = $('<iframe>', {
            src: '$currentUrl',
            id: 'editor-frame'
        });
        $('nav').before(iframe);
        iframe.css('height', 'calc(100vh - ' + $('nav').height() + 'px)');
        iframe.css('width', '100%');

        $(document).scroll(function() {
            $(this).scrollTop(0);
        });

        $('body').addClass('editor');
    })(jQuery);
}

SqueezeBox.extend({
    applyContent: function(content, size) {
        if (!this.isOpen && !this.applyTimer) return;
        this.applyTimer = clearTimeout(this.applyTimer);
        this.hideContent();
        if (!content) {
            this.toggleLoading(true);
        } else {
            if (this.isLoading) this.toggleLoading(false);
            this.fireEvent('onUpdate', [this.content], 20);
        }
        if (content) {
            if (['string', 'array'].contains(typeOf(content))) {
                this.content.set('html', content);
            } else if (!(content !== this.content && this.content.contains(content))) {
                this.content.adopt(content);
            }
        }
        this.callChain();
        if (!this.isOpen) {
            this.toggleListeners(true);
            this.resize(size, true);
            this.isOpen = true;
            this.win.setProperty('aria-hidden', 'false');
            this.fireEvent('onOpen', [this.content]);
        } else {
            this.resize(size);
        }
    }
});

function sendRequest(data, callback) {
    var xhr = new XMLHttpRequest();

    function onError() {
        callback(new Error('Failed to send a request to ' + data.url + ' ' + JSON.stringify({
            responseText: xhr.responseText,
            readyState: xhr.readyState,
            status: xhr.status
        }, null, 4)));
    }

    xhr.onerror = onError;
    xhr.onload = function () {
        if (this.readyState === 4 && this.status === 200) {
            callback(null, this.response);
        } else {
            onError();
        }
    };
    xhr.open(data.method || 'GET', data.url);

    if (data.data) {
        var formData = new FormData();
        formData.append("pageType", data.data.pageType);
        formData.append("pageId", data.data.pageId);
        xhr.send(formData);
    } else {
        xhr.send();
    }
}

function postMessageListener(event) {
    if (event.origin !== location.origin) {
        return;
    }
    var data = JSON.parse(event.data);
    if (!data) {
        return;
    }
    if (data.action === 'close') {
        window.location.href = data.closeUrl;
    } else if (data.action === 'editLinkDialogOpen') {
        openEditLinkDialog(data.data);
    }
}

if (window.addEventListener) {
    window.addEventListener("message", postMessageListener);
} else {
    window.attachEvent("onmessage", postMessageListener); // IE8
}
EOF;
    }

    /**
     * Add script for making data for editor
     */
    public function addDataBridgeScript()
    {
        $app = JFactory::getApplication();
        $input = $app->input;
        $aid = $this->_article->id;
        $start = $input->get('start', '0');
        $autostart = $input->get('autostart', '0');
        $domain = $this->getDomain();

        $editorSettings = NicepageHelpersNicepage::getEditorSettings();
        $editorSettings['pageId'] = $this->_isConvertRequired ? '' : $aid;
        $editorSettings['startPageId'] = $aid;

        $cmsSettings = NicepageHelpersNicepage::getCmsSettings();
        $cmsSettings['isFirstStart'] = $start == '1' ? true : false;

        $editorSettingsJson = json_encode($editorSettings, JSON_PRETTY_PRINT);
        $cmsSettingsJson = json_encode($cmsSettings, JSON_PRETTY_PRINT);

        $modelActions = new NicepageModelsActions();
        $site = $modelActions->getSite();
        if ($this->_article->state == '2' && ($start == '1' || $autostart == '1')) {
            $site['items'][] = array(
                'siteId' => '1',
                'title' => '',
                'id' => (string) $aid,
                'order' => 0,
                'status' => 2,
                'editorUrl' => $this->getAdminUrl() . '/index.php?option=com_nicepage&autostart=1&postid=' . $aid . ($domain ? '&domain=' . $domain : ''),
                'htmlUrl' => $this->getAdminUrl() . '/index.php?option=com_nicepage&controller=actions&action=getPageHtml&pageId=' . $aid
            );
        }
        $info = array(
            'productsExists' => $this->vmEnabled(),
            'newPageUrl' => $this->getAdminUrl() . '/index.php?option=com_nicepage&dashboard=1' . ($domain ? '&domain=' . $domain : '')
        );
        $themeEditorSettings = $this->getEditorSettingsFromDefaultTheme();
        if ($themeEditorSettings) {
            $info['themeTypography'] = $themeEditorSettings['typography'];
            $info['themeFontScheme'] = $themeEditorSettings['fontScheme'];
            $info['themeColorScheme'] = $themeEditorSettings['colorScheme'];
        }

        $data = json_encode(
            array (
                'site' => $site,
                'pagePosts' => $modelActions->getSitePostsResult(array('options' => array('pageId' => $aid))),
                'pageHtml' => $modelActions->getPageHtml($aid),
                'startTerm' => $this->_isConvertRequired ? 'site:joomla:' . $aid : '',
                'info' => $info,
                'pageUrlForParsing' => $this->getFrontendUrl(),
                'nicePageCss' => $this->getDynamicNicepageCss()
            ),
            JSON_PRETTY_PRINT
        );

        $this->_scripts .= <<<EOF
var dataBridgeData = $data;
window.dataBridge = {
    getSite: function () {
        return dataBridgeData.site;
    },
    getSitePosts: function () {
        return dataBridgeData.pagePosts;
    },
    getPageHtml: function () {
        return dataBridgeData.pageHtml;
    },
    getStartTerm: function () {
        return dataBridgeData.startTerm;
    },
    getInfo: function getInfo() {
        return dataBridgeData.info;
    },
    getNicePageCss: function getNicePageCss() {
        return dataBridgeData.nicePageCss;
    },
    settings: $editorSettingsJson,
    cmsSettings: $cmsSettingsJson
};
EOF;
    }

    /**
     * Add joomla link dialog script
     */
    public function addLinkDialogScript()
    {
        $editLinkUrl = $this->getAdminUrl() . '/index.php?option=com_content&view=articles&layout=modal&tmpl=component';
        $customUrlOptions = <<<HTML
<style>
.custom-url-options {
    width:100%;
}
.custom-url-options label{
    width: 60px;
    display: inline-block;
}
.custom-url-options input[type=text]{
    width: 350px;
}
.custom-url-options:after {
    content: "";
    clear: both;
    display: table;
}
.target-option {
    margin-left: 70px;
}
.target-option input {
    margin-right: 10px;
}
</style>
<div class="custom-url-options">
    <div style="float:left">
        <div class="caption-option"><label for="caption">{{caption}}</label><input type="text" name="caption" value="" /></div>
        <div class="url-option"><label for="url">{{url}}</label><input type="text" name="url" value="" /></div>
        <div class="target-option"><input type="checkbox" name="target" />{{target}}</div>
    </div>
    <div style="float:right">
        <button type="button" class="btn btn-success" id="save-options">Save</button>
    </div>
</div>
HTML;
        $customUrlOptions = base64_encode($customUrlOptions);

        $this->_scripts .= <<<EOF
function openEditLinkDialog(data) {
    //override joomla action for modal close
    window.jModalClose = function () {};
    window.jSelectArticle = function () {};
    (function($) {
        var editorFrame = $('#editor-frame')[0].contentWindow;
        var editLinkUrl = '$editLinkUrl' + '&u=' + Date.now();
        SqueezeBox.fromElement(editLinkUrl, {
            size : {x : 800, y : 500},
            iframePreload: true,
            handler : 'iframe',
            onOpen : function (container, showContent, data) {
                if (container.firstChild) {
                    var ifrDoc = container.firstChild.contentDocument;
                    if (!$('.custom-url-options', ifrDoc).length && showContent && data) {
                        var customHtml = atob('$customUrlOptions').replace(/\{\{(\w+?)\}\}/g, function(str, p1) {
                            return data.l[p1];
                        });
                        $('body', ifrDoc).prepend(customHtml);
                        $('.caption-option input', ifrDoc).val(data.value ? data.value : 'Text');
                        $('.url-option input', ifrDoc).val(data.url ? data.url : '#');
                        $('.target-option input', ifrDoc).prop('checked', data.blank);
                    }
                    $('#adminForm table tbody a', ifrDoc).on('click', function() {
                        var uriAttr = $(this).attr('data-uri') || $(this).attr('onclick'),
                            url = uriAttr.match(/index.php[^,'"]+/),
                            text = $(this).text().trim();
                        $('.caption-option input', ifrDoc).val(text);
                        $('.url-option input', ifrDoc).val(url ? url[0] : '');
                    });
                    
                    $('#save-options', ifrDoc).on('click', function() {
                        $(ifrDoc).data('close-after-save', true);
                        editorFrame.postMessage(JSON.stringify({
                            action: 'editLinkDialogClose',
                            data: {
                                value: $('.caption-option input', ifrDoc).val(),
                                url: $('.url-option input', ifrDoc).val(),
                                blank: $('.target-option input', ifrDoc).is(":checked") ? true : false
                            }
                        }), window.location.origin);
                        SqueezeBox.close();
                    });
                    container.setStyle('display', showContent ? '' : 'none');
                }
            },
            onClose : function (container) {
                if (!$(container.firstChild.contentDocument).data('close-after-save'))
                    editorFrame.postMessage(JSON.stringify({action: 'editLinkDialogClose'}), window.location.origin);
            }
        });
        window.setTimeout(function () {
            SqueezeBox.fireEvent('onOpen', [SqueezeBox.content, true, data]);
        }, 1000);
    })(jQuery);
}            
EOF;

    }

    /**
     * Include all scripts to page document
     */
    public function includeScripts()
    {
        $doc = JFactory::getDocument();

        $parserUrl = $this->getAdminUrl() . '/components/com_nicepage/assets/js/typography-parser.js';
        $doc->addScript($parserUrl);

        $doc->addScriptDeclaration($this->_scripts);
    }

    /**
     * Display nicepage editor button
     */
    public function displayEditButton()
    {
        $input = JFactory::getApplication()->input;
        $start = $input->get('start', '0');
        $autostart = $input->get('autostart', '0');
        $favicon = $this->getAdminUrl() . '/components/com_nicepage/assets/images/button-icon.png';
        $cssDisplay = ($start == '1' || $autostart == '1') ? 'none' : 'block';
        $css = <<<EOF
body.editor>*:not(nav){
    display:none;
}

body>*:not(nav) {
    display:$cssDisplay;
}

#np-loader {
	display: block !important;
}

#editor-frame{
    display: block !important;
}
#sbox-overlay, #sbox-window  {
    display: block;
}
.btn.nicepage-button {
    color: #4184F4;
    background: url('$favicon') no-repeat 4px 4px;
    font-weight: bold;
    font-family: Arial;
    padding-right: 5px;
    padding-left: 25px;
    margin: 10px 10px 20px 0px;
    background-size: 16px;
}
.nicepage-select-template {
    margin-left: 10px
}
.nicepage-select-template-area {
    border: 1px solid #ddd;
    margin: 10px 0 20px 0;
    padding: 15px;
}
EOF;
        JFactory::getDocument()->addStyleDeclaration($css);
    }

    /**
     * Display screenshot of article, created from sections
     */
    public function displayScreenshots()
    {
        if (!$this->_sections) {
            return;
        }

        $content = NicepageModelsSections::getSectionsScreenshots($this->_sections);
        $css = <<<EOF
fieldset.adminform {
    display:none;
}
.nicepage-button {
    margin: 20px;
}
#preview-container {
    overflow: hidden;
}
#preview-frame {
    transform: scale(0.3);
    transform-origin: 0 0;
    height: 333.333%;
}
EOF;
        $js = <<<EOF
function renderScreenshots()
{
    (function($){
        var previewContainer = $('<div>', {
            id: 'preview-container'
        }),
        previewFrame = $('<iframe>', {
            id: 'preview-frame',
            frameborder: 0,
            scrolling: 'no',
            width: '9999px'
        });

        $('.adminform').before(previewContainer);
        previewContainer.append(previewFrame);
       
        var doc = previewFrame[0].contentDocument;
        doc.open();
        doc.write($content);
        doc.close();
        
        previewFrame.load(function() {
            var containerHeight = 0;
            $('section', previewFrame.contents()).each(function(i, el) {
                containerHeight += $(el).height();
            });
            previewContainer.height(containerHeight * 0.3);
        });

        $('body', previewFrame.contents()).click(function(e) {
            runNicepage();
        });
    })(jQuery);
}

jQuery(function($) {
    renderScreenshots();
});
EOF;
        $doc = JFactory::getDocument();
        $doc->addScriptDeclaration($js);
        $doc->addStyleDeclaration($css);
    }

    /**
     * Check nicepage component
     *
     * @return bool
     */
    public function nicepageComponentInstalled()
    {
        if (!file_exists(dirname(JPATH_ADMINISTRATOR) . '/components/com_nicepage')) {
            return false;
        }

        if (!JComponentHelper::getComponent('com_nicepage', true)->enabled) {
            return false;
        }
        return true;
    }

    /**
     * Check the existence of Virtuemart
     *
     * @return bool
     */
    public function vmEnabled()
    {
        if (!file_exists(dirname(JPATH_ADMINISTRATOR) . '/components/com_virtuemart/')) {
            return false;
        }

        if (!JComponentHelper::getComponent('com_virtuemart', true)->enabled) {
            return false;
        }
        return true;
    }

    /**
     * Set domain property
     */
    public function setDomain()
    {
        $this->_domain = JRequest::getVar('domain', (defined('NICEPAGE_DOMAIN') ? NICEPAGE_DOMAIN : ''));
    }

    /**
     * Get domain property
     *
     * @return string
     */
    public function getDomain()
    {
        return $this->_domain;
    }

    /**
     * Set admin url
     */
    public function setAdminUrl()
    {
        $current = dirname(dirname((JURI::current())));
        $this->_adminUrl = $current . '/administrator';
    }

    /**
     * Get admin url
     *
     * @return string
     */
    public function getAdminUrl()
    {
        return $this->_adminUrl;
    }

    /**
     * Get name of default template style
     *
     * @return string
     */
    public function getDefaultTemplate()
    {
        $db = JFactory::getDBO();
        $query = $db->getQuery(true);
        $query->select('*');
        $query->from('#__template_styles');
        $query->where('client_id = \'0\'');
        $query->where('home=1');
        $db->setQuery($query);
        $ret = $db->loadObject();
        return $ret ? $ret->template : '';
    }

    /**
     * Get editor settings from default theme
     *
     * @return mixed|null
     */
    public function getEditorSettingsFromDefaultTheme()
    {
        $template = $this->getDefaultTemplate();
        if ($template) {
            $funcsFilePath = dirname(dirname(JPATH_THEMES)) . '/templates/' . $template . '/template.json';
            if (file_exists($funcsFilePath)) {
                ob_start();
                include_once $funcsFilePath;
                return json_decode(ob_get_clean(), true);
            }
        }
        return null;
    }

    /**
     * Get frontend site url
     *
     * @return string
     */
    public function getFrontendUrl()
    {
        $db = JFactory::getDbo();
        $query = $db->getQuery(true);
        $query->select('*')
            ->from($db->quoteName('#__content'))
            ->where($db->quoteName('state') . ' = 1');
        $db->setQuery($query);
        $ret = $db->loadObject();

        if (false && $ret !== null) {
            return dirname($this->getAdminUrl()) . '/' . 'index.php?option=com_content&view=article&id=' . $ret->id . '&toEdit=1';
        } else {
            $frontEndUri = JUri::getInstance(JUri::root());
            $frontEndUri->setScheme(((int) JFactory::getApplication()->get('force_ssl', 0) === 2) ? 'https' : 'http');
            $frontEndUri->setVar('toEdit', '1');
            return $frontEndUri->toString();
        }
    }

    /**
     * Get content from nicepage-dynamic.css
     *
     * @return string
     */
    public function getDynamicNicepageCss()
    {
        $assets = dirname(JPATH_PLUGINS) . '/administrator/components/com_nicepage/assets';
        ob_start();
        include $assets . '/css/nicepage-dynamic.css';
        return ob_get_clean();
    }
}